@@ -30,7 +30,7 @@
 #include "saphire_extra.h"
 
 /// from readline.c ///
-void readline_init();
+void readline_init(BOOL run_user_runtimescript);
 void readline_final();
 
 //////////////////////////////////////////////////////////////////////
@@ -368,7 +368,7 @@ int main(int argc, char* argv[])
 
     saphire_init(kATConsoleApp, TRUE, runtime_script, TRUE);
     saphire_job_done = main_saphire_job_done;
-    readline_init();
+    readline_init(runtime_script);
     saphire_rehash();
 
     /// ヒストリーを読み込む ///
