@@ -64,6 +64,7 @@ static BOOL name_sort(void* left, void* right)
 
 
 static string_obj* gEditingDir;
+static string_obj* gEditingDirQuote;
 static BOOL read_one_argument(char* p, string_obj* buf)
 {
     /// 単語の区切り記号かどうかのテーブル 127以下なら
@@ -71,7 +72,7 @@ static BOOL read_one_argument(char* p, string_obj* buf)
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
@@ -813,255 +814,13 @@ char* readline_method_completion(const char* text, int stat)
     return NULL;
 }
 
-char* readline_sweep_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static hash_it* it2;
-    static hash_it* it3;
-    static hash_it* it4;
-    static hash_it* it5;
-    static hash_it* it6;
-    static hash_it* it7;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gArrays);
-        it2 = hash_loop_begin(gHashs);
-        it3 = hash_loop_begin(gGlobals);
-        it4 = hash_loop_begin(gFuncs);
-        it5 = hash_loop_begin(gClasses);
-        it6 = hash_loop_begin(gObjects);
-        it7 = hash_loop_begin(gRefs);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    while(it2) {
-        char* key = hash_loop_key(it2);
-        it2 = hash_loop_next(it2);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    while(it3) {
-        char* key = hash_loop_key(it3);
-        it3 = hash_loop_next(it3);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    while(it4) {
-        char* key = hash_loop_key(it4);
-        it4 = hash_loop_next(it4);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    while(it5) {
-        char* key = hash_loop_key(it5);
-        it5 = hash_loop_next(it5);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    while(it6) {
-        char* key = hash_loop_key(it6);
-        it6 = hash_loop_next(it6);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    while(it7) {
-        char* key = hash_loop_key(it7);
-        it7 = hash_loop_next(it7);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-char* readline_ary_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gArrays);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-char* readline_hash_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gHashs);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-char* readline_global_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gGlobals);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-char* readline_def_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gFuncs);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-char* readline_class_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gClasses);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-char* readline_object_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gObjects);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-typedef struct {
-    regex_t* mRegex;
-    sStatments* mStatments;
-} sUserRegex;
-
-vector_obj* gUserCompRegex;
-
-sUserRegex* sUserRegex_new(regex_t* regex, sStatments* statments)
-{
-    sUserRegex* self = MALLOC(sizeof(sUserRegex));
-
-    self->mRegex = regex;
-    self->mStatments = STATMENTS_NEW2(statments);
-
-    return self;
-}
-
-void sUserRegex_delete(sUserRegex* self)
-{
-    sStatments_delete(self->mStatments);
-    FREE(self);
-}
 
 static vector_obj* gUserCompletions;
 
 static string_obj* gUserCompletionStatment;
 static sStatments* gUserCompletionStatments;
 
-char* readline_completion(const char* text, int stat)
+char* readline_user_completion(const char* text, int stat)
 {
     static int index, wordlen;
 
@@ -1151,7 +910,7 @@ char* readline_deref_completion(const char* text, int stat)
         it = hash_loop_next(it);
 
         if(!strncmp(text + 1, key, wordlen-1)) {
-            if(gDerefCompletionProgram && ref->mKind == kMCObject) {
+            if(gDerefCompletionProgram && ref->mKind == kMCObject & memchecker_is_enable_mem(ref->mMem) == kMCObject) {
                 char buf[1024];
                 snprintf(buf, 1024, "$%s->", key);
                 rl_completion_append_character= 0;
@@ -1169,51 +928,12 @@ char* readline_deref_completion(const char* text, int stat)
     return NULL;
 }
 
-char* readline_ref_completion(const char* text, int stat)
-{
-    static hash_it* it;
-    static int wordlen;
-
-    if(stat == 0) {
-        wordlen = strlen(text);
-        it = hash_loop_begin(gRefs);
-    }
-
-    while(it) {
-        char* key = hash_loop_key(it);
-        it = hash_loop_next(it);
-
-        if(!strncmp(text, key, wordlen)) {
-            return strdup(key);
-        }
-    }
-
-    return NULL;
-}
-
-
 char* readline_file_completion(const char* text, int rstat)
 {
-    /// クォートを除外 ///
-    char* text2 = MALLOC(strlen(text) +1);
-    char* _p = (char*)text;
-    char* _p2 = text2;
-
-    while(*_p) {
-        if(*_p == '\\') {
-            _p++;
-        }
-        else {
-            *_p2++ = *_p++;
-        }
-    }
-    *_p2 = 0;
-
-    static int index, wordlen;
+    static int index;
 
     /// 初期化 ///
     if(rstat == 0) {
-        wordlen = strlen(text2);
         index = 0;
 
         // ディレクトリのパートを得る
@@ -1266,6 +986,9 @@ char* readline_file_completion(const char* text, int rstat)
             string_push_back(gEditingDir2, p);
 
             string_put(gEditingDir, string_c_str(gEditingDir2));
+
+            string_delete(user);
+            string_delete(gEditingDir2);
         }
 
         /// 補完候補を初期化 ///
@@ -1381,7 +1104,7 @@ char* readline_file_completion(const char* text, int rstat)
         index++;
 
         /// ファイル名の入力がない補完は判定しなくても全てマッチ ///
-        if(strcmp(text2, "") == 0) {
+        if(strcmp(text, "") == 0) {
             char path[PATH_MAX];
             xstrncpy(path, file, PATH_MAX);
 
@@ -1402,13 +1125,27 @@ char* readline_file_completion(const char* text, int rstat)
                 rl_completion_append_character= ' ';
             }
 
-            FREE(text2);
             return ret;
         }
 
         /// アスキー文字同士の補完 ///
         else {
-            if(!strncmp(text2, file, wordlen)) {
+            /// クォートを除外 ///
+            char* text_noquote = MALLOC(strlen(text) +1);
+            char* _p = (char*)text;
+            char* _p2 = text_noquote;
+
+            while(*_p) {
+                if(*_p == '\\') {
+                    _p++;
+                }
+                else {
+                    *_p2++ = *_p++;
+                }
+            }
+            *_p2 = 0;
+
+            if(!strncmp(text_noquote, file, strlen(text_noquote))) {
                 char path[PATH_MAX];
                 xstrncpy(path, file, PATH_MAX);
 
@@ -1431,44 +1168,24 @@ char* readline_file_completion(const char* text, int rstat)
                     rl_completion_append_character= ' ';
                 }
 
-                FREE(text2);
+                FREE(text_noquote);
                 return ret; 
             }
+
+            FREE(text_noquote);
         }
     }
 
     if(index == 0 && vector_size(files) == 0) {
         index ++;
         rl_completion_append_character= 0;
-        FREE(text2);
         return strdup("");
     }
 
-    FREE(text2);
     return NULL;
 }
 
-static regex_t* get_reg(int* r, char* regex2)
-{
-    regex_t* reg;
-    OnigEncoding enc = ONIG_ENCODING_UTF8;
-    OnigErrorInfo err_info;
-    *r = onig_new(&reg, regex2
-            , regex2 + strlen((char*)regex2)
-            , ONIG_OPTION_DEFAULT
-            , enc
-            //, ONIG_SYNTAX_RUBY
-            //, ONIG_SYNTAX_PERL_NG
-            , ONIG_SYNTAX_DEFAULT
-            , &err_info);
-
-    if(*r != ONIG_NORMAL) {
-        onig_free(reg);
-    }
-
-    return reg;
-}
-
+hash_obj* gUserCompRegex;
 
 char** readline_on_complete(const char* text, int start, int end)
 {
@@ -1649,7 +1366,7 @@ char** readline_on_complete(const char* text, int start, int end)
                 if(string_c_str(buf)[0] == '$') {
                     sRef* ref = saphire_get_ref(string_c_str(buf) + 1);
 
-                    if(ref && ref->mKind == kMCObject) {
+                    if(ref && ref->mKind == kMCObject && memchecker_is_enable_mem(ref->mMem) == kMCObject) {
                         object = ref->mMem;
                     }
                     else {
@@ -1851,15 +1568,13 @@ char** readline_on_complete(const char* text, int start, int end)
             string_put(buf, "");
         }
         else if(*p == '/') {
+            string_push_back2(buf, *p);
             user = FALSE;
-            program = FALSE;
+            if(program) { string_push_back2(command, *p); }
             method = FALSE;
             ref = FALSE;
-            if(program) { string_push_back2(command, *p); }
             string_push_back2(statment, *p);
             p++;
-
-            string_put(buf, "");
         }
         /// 終了 ///
         else if(*p == 0) {
@@ -1895,39 +1610,26 @@ char** readline_on_complete(const char* text, int start, int end)
         }
     }
 
-    sStatments* statments = NULL;
-    if(strcmp(string_c_str(buf), string_c_str(command)) != 0) {
-        int i;
-        for(i=0; i<vector_size(gUserCompRegex); i++) {
-            sUserRegex* ureg = vector_item(gUserCompRegex, i);
-
-            if(ureg) {
-                regex_t* reg = ureg->mRegex;
-                char* str2 = string_c_str(command);
-
-                OnigRegion* region = onig_region_new();
-                int r2 = onig_search(reg, str2
-                   , str2 + strlen(str2)
-                   , str2, str2 + strlen(str2)
-                   , region, ONIG_OPTION_NONE);
-                onig_region_free(region, 1);
-
-                if(r2 >= 0) {
-                    statments = ureg->mStatments;
-                    break;
-                }
-            }
-        }
+    sStatments* statments;
+    /// 最後の単語入力が=が入っている場合ファイル名の補完をしたいのでカスタマイズ補完は無効にしたい
+    if(strstr(string_c_str(buf), "=")) {
+        statments = NULL;
     }
+    else if(strcmp(string_c_str(buf), string_c_str(command)) != 0) {
+        statments = hash_item(gUserCompRegex, string_c_str(command));
+    }
+    else {
+        statments = NULL;
+    }
+
+    if(strstr(string_c_str(command), "/")) {
+        program = FALSE;
+    }
+
     string_delete(command);
     string_delete(buf);
 
-    if(statments) {
-        gUserCompletionStatment = statment;
-        gUserCompletionStatments = statments;
-        return rl_completion_matches(text, readline_completion);
-    }
-    else if(user) {
+    if(user) {
         rl_completion_append_character= 0;
         string_delete(statment);
         return rl_completion_matches(text, rl_username_completion_function);
@@ -1947,6 +1649,11 @@ char** readline_on_complete(const char* text, int start, int end)
         string_delete(statment);
         return rl_completion_matches(text, readline_program_completion);
     }
+    else if(statments) {
+        gUserCompletionStatment = statment;
+        gUserCompletionStatments = statments;
+        return rl_completion_matches(text, readline_user_completion);
+    }
 #if defined(HAVE_MIGEMO_H)
     else {
         string_delete(statment);
@@ -2133,26 +1840,49 @@ BOOL commands_remove_history(int* rcode, vector_obj* argv, vector_obj* blocks, v
     return TRUE;
 }
 
+BOOL commands_call_completion(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    if(vector_size(argv) >= 2) {
+        string_obj* command = vector_item(argv, 1);
+
+        sStatments* statments;
+        if(string_c_str(command) != 0) {
+            statments = hash_item(gUserCompRegex, string_c_str(command));
+        }
+        else {
+            statments = NULL;
+        }
+        if(statments) {
+            int rcode = saphire_run(statments, "user completion", nextout, nextin,  nexterr);
+            if(rcode >= 0) {
+                return TRUE;
+            }
+        }
+        else {
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
 
 BOOL commands_completion(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
 {
     if(vector_size(argv) >= 2 && vector_size(blocks) == 1) {
+        sBlock* block = vector_item(blocks, 0);
+        sStatments* statments = block->mStatments;
+
         int i;
         for(i=1; i<vector_size(argv); i++) {
-            char* regex = string_c_str(vector_item(argv, i));
-            sBlock* block = vector_item(blocks, 0);
-            sStatments* statments = block->mStatments;
+            char* command = string_c_str(vector_item(argv, i));
 
-            int r;
-            regex_t* reg = get_reg(&r, regex);
-            if(r == ONIG_NORMAL) {
-                vector_add(gUserCompRegex, sUserRegex_new(reg, statments));
-                *rcode = 0;
-            }
-            else {
-                err_msg("invalid regex", sname, sline);
-                return FALSE;
+            sStatments* statments2 = hash_item(gUserCompRegex, command);
+            if(statments2) {
+                sStatments_delete(statments2);
             }
+            hash_put(gUserCompRegex, command, STATMENTS_NEW2(statments));
+
+            *rcode = 0;
         }
     }
 
@@ -2208,9 +1938,9 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
     }
 }
 
-void readline_init()
+void readline_init(BOOL run_user_runtimescript)
 {
-    gUserCompRegex = VECTOR_NEW(100);
+    gUserCompRegex = HASH_NEW(100);
     gUserCompletions = VECTOR_NEW(50);
 
     gUserCompletionStatment = NULL;
@@ -2219,7 +1949,7 @@ void readline_init()
     rl_attempted_completion_function = readline_on_complete;
     rl_completer_quote_characters = "\"'";
     rl_completer_word_break_characters = " \t\n\"'|{}!&;()<>=%/";
-    rl_filename_quote_characters = " \\\t\n\"|$*?[]<>{}&;#()'%~";
+    rl_filename_quote_characters = " \\\t\n\"|$*?[]<>{}&;#()'%";
     rl_completion_append_character= ' ';
     rl_char_is_quoted_p = (rl_linebuf_func_t*)lftp_char_is_quoted;
 
@@ -2228,6 +1958,7 @@ void readline_init()
 #endif
 
     gEditingDir = STRING_NEW("");
+    gEditingDirQuote = STRING_NEW("");
     gMethodCompletions = NULL;
 
     rl_bind_key('x'-'a'+1, readline_macro);
@@ -2240,7 +1971,10 @@ void readline_init()
     saphire_add_inner_command("clear_history", commands_clear_history);
     saphire_add_inner_command("remove_history", commands_remove_history);
     saphire_add_inner_command("completion", commands_completion);
-    read_rc_obj_file(TRUE);
+    saphire_add_inner_command("call_completion", commands_call_completion);
+    if(run_user_runtimescript) {
+        read_rc_obj_file(TRUE);
+    }
 
     files = VECTOR_NEW(30);
 }
@@ -2254,11 +1988,14 @@ void readline_final()
         }
         vector_delete(gMethodCompletions);
     }
-    int l;
-    for(l=0; l<vector_size(gUserCompRegex); l++) {
-        sUserRegex_delete(vector_item(gUserCompRegex, l));
+
+    hash_it* it = hash_loop_begin(gUserCompRegex);
+    while(it) {
+        sStatments_delete(hash_loop_item(it));
+
+        it = hash_loop_next(it);
     }
-    vector_delete(gUserCompRegex);
+    hash_delete(gUserCompRegex);
 
     int k;
     for(k=0; k<vector_size(gUserCompletions); k++) {
@@ -2273,6 +2010,7 @@ void readline_final()
     }
     vector_delete(files);
     string_delete(gEditingDir);
+    string_delete(gEditingDirQuote);
 #if defined(HAVE_MIGEMO_H)
     migemo_final();
 #endif
