@@ -11,7 +11,7 @@ enum eCommandKind {
     kObject, kRef, kVar, kGlobal, kExport, 
     kAry, kAryAdd, kAryErase, 
     kHash, kHashAdd, kHashErase,  
-    kDef, kInherit, kSelf, kClass,
+    kDef, kInherit, kSelf, kStackFrame, kClass,
     kSweep, kShow,
     kSubshell, kMSleep, kTrue, kFalse, kTest,
     kIndex, kRIndex, kLength, kLc, kUc,
@@ -19,7 +19,7 @@ enum eCommandKind {
     kJobs, kRehash, kKanjiCode, kLineField,
     kCalc, kPrint,  kPuts, kLoad, 
     kWhile, kBreak, kExit, kIf, kSplit, kX, kJoin, kSelect, kLines, kRows,
-    kMatch, kScan, kErase, kSub,
+    kMatch, kScan, kErase, kSub, kGSub, 
     kRead, kClose, 
     kCd, 
     kAdd, kDel, kSelector, kMax, kMin,
